@@ -24,13 +24,25 @@ def dataset_grid(api: API) -> None:
     for viewing. In viewing mode, the dataset can be edited or deleted. The grid
     does also include a button for creating new datasets
     """
+    # create the select interface
+    st.sidebar.markdown('## Filter')
+    use_field = st.sidebar.checkbox('Random Fields', value=True)
+    use_sample = st.sidebar.checkbox('Spatial Sample', value=True)
+    use_aux = st.sidebar.checkbox('Auxiliary data', value=False)
+    
+    # build a filter
+    filt = [name for name, use in zip(('field', 'sample', 'auxiliary'), (use_field, use_sample, use_aux)) if use]
+    
     # get the upload names
-    all_datasets = api.get_upload_names()
+    all_datasets = api.get_upload_names(data_type=filt)
 
     # get the options
     options = card_options_from_dataset_names(api, all_datasets)
 
     # check for data_id
+    st.title('Datasets')
+    st.markdown("""""")
+
     data_id = card_select(options=options, spacing=5, md=3, lg=3)
 
     # if None selected, stop
@@ -46,9 +58,14 @@ def dataset_grid(api: API) -> None:
         st.experimental_rerun()
 
 
-def button_panel(can_resample: bool = False, container=st) -> None:
+def button_panel(can_resample: bool = False, can_upload: bool = False, container=st) -> None:
     # build the columns in the container
-    cols = container.columns(4 if can_resample else 3)
+    n_col = 3
+    if can_resample:
+        n_col += 1
+    if can_upload:
+        n_col += 1
+    cols = container.columns(n_col)
 
     # add the buttons
     back = cols[0].button('BACK TO LIST')
@@ -59,6 +76,11 @@ def button_panel(can_resample: bool = False, container=st) -> None:
         resample = cols[2].button('RE-SAMPLE DATASET')
     else:
         resample = False
+    
+    if can_upload:
+        aux = cols[3 if can_resample else 2].button('UPLOAD AUXILIARY')
+    else:
+        aux = False
 
     # check the action
     if back:
@@ -74,6 +96,9 @@ def button_panel(can_resample: bool = False, container=st) -> None:
     elif resample:
         st.session_state.action = 'sample'
         st.experimental_rerun()
+    elif aux:
+        st.session_state.action = 'auxiliary'
+        st.experimental_rerun()
 
 
 def action_view(api: API) -> None:
@@ -87,7 +112,7 @@ def action_view(api: API) -> None:
 
     # button list
     button_expander = st.expander('ACTIONS', expanded=True)
-    button_panel(can_resample=dataset.data_type == 'field', container=button_expander)
+    button_panel(can_resample=dataset.data_type == 'field', can_upload=dataset.data_type in ('field', 'sample'), container=button_expander)
 
     # main description
     left, right = st.columns((6, 4))
@@ -103,7 +128,7 @@ def action_view(api: API) -> None:
         right.info('Edit the dataset to add the origin')
 
     st.markdown('### Preview')
-    components.dataset_plot(dataset, disable_download=False)
+    components.dataset_plot(dataset, disable_download=False, add_controls=True)
 
     # debug area
     exp = st.expander('RAW database record')
@@ -113,28 +138,25 @@ def action_view(api: API) -> None:
 def upload_view(api: API) -> None:
     # Title
     st.title('Upload a new dataset')
+
+    # back button
+    go_back = st.sidebar.button('Back to List')
+    if go_back:
+        del st.session_state['action']
+        st.experimental_rerun()
+
     st.info('As of now, the Dataset will be named exactly like the uploaded file. If you would like to change the name, you need to edit the dataset afterwards.')
     
     # upload handler
     dataset = components.upload_handler(api=api, can_select=False)
 
-    # add the preview
-    dataset.update_thumbnail()
-
-    # add auxiliary data
-    components.upload_auxiliary_data(dataset=dataset, api=api)
-
-    # preview
-    st.markdown('## Preview upload')
-    components.dataset_plot(dataset, disable_download=True)
-
-    st.markdown('## Finished?')
-    go_back = st.button('Back')
-    if go_back:
-        st.session_state.action = 'list'
+    if dataset is not None:
+        st.session_state.data_id = dataset.id
+        st.session_state.action = 'view'
         st.experimental_rerun()
 
 
+
 def edit_view(api: API) -> None:
     # get the dataset
     dataset = api.get_upload_data(id=st.session_state.data_id)
@@ -203,9 +225,48 @@ def sample_view(api: API) -> None:
     st.title(f'Re-Sample {dataset.upload_name}')
     st.markdown('Use this little sub-app to create a new sample from the selected dense dataset or field. The new sample can be used just like any other dataset')
 
+    # buttons
+    with st.sidebar.expander('ACTIONS', expanded=False):
+        l, r = st.columns(2)
+        go_list = l.button('Back to List')
+        go_data = r.button(f"Back to {dataset.upload_name}")
+
+        if go_list:
+            del st.session_state['action']
+            st.experimental_rerun()
+        if go_data:
+            st.session_state.action = 'view'
+            st.session_state.data_id = dataset.id
+            st.experimental_rerun()
+
     sample_dense_data(dataset=dataset, api=api)
 
 
+def auxiliary_upload_view(api: API) -> None:
+    # ger the dataset
+    dataset = api.get_upload_data(id=st.session_state.data_id)
+
+    # Title
+    st.title('Upload auxiliary information')
+    st.markdown(f"### for {dataset.upload_name}")
+
+    # buttons
+    with st.sidebar.expander('ACTIONS', expanded=False):
+        l, r = st.columns(2)
+        go_list = l.button('Back to List')
+        go_data = r.button(f"Back to {dataset.upload_name}")
+
+        if go_list:
+            del st.session_state['action']
+            st.experimental_rerun()
+        if go_data:
+            st.session_state.action = 'view'
+            st.session_state.data_id = dataset.id
+            st.experimental_rerun()
+    
+    components.upload_auxiliary_data(dataset=dataset, api=api)
+
+
 def sample_dense_data(dataset: DataUpload, api: API):
     # create the sidebar
     st.sidebar.title('Parameters')
@@ -363,32 +424,13 @@ def edit_dataset(dataset: DataUpload, api: API, container = st) -> None:
         st.session_state.action = 'view'
         st.experimental_rerun()
 
-    # extract the data
-    data = dataset.data
+    # use the edit dataset component
+    dataset = components.edit_dataset(dataset, api, container=container)
 
-    # Build the main form
-    with container.form('EDIT'):
-        LIC = components.utils.LICENSES
-        new_title= st.text_input('Title', dataset.upload_name)
-        new_origin = st.text_area('Origin', value=data.get('origin', ''), help="Add the source of the dataset to help others cite it correctly.")
-        new_description = st.text_area('Description', value=data.get('description', ''), help="Add a description of the dataset.")
-        new_license = st.selectbox('License', options=list(LIC.keys()), index=list(LIC.keys()).index(data.get('license', 'ccby')), format_func=lambda k: LIC.get(k))
-        save = st.form_submit_button('SAVE')
-    
-        # check save
-        if save:
-            updates = {'license': new_license}
-            if new_origin.strip() != '':
-                updates['origin'] = new_origin
-            if new_description.strip() != '':
-                updates['description'] = new_description
-            
-            # overwrite dataset 
-            dataset = api.update_upload_data(id=dataset.id, name=new_title, **updates)
-            
-            # switch back to view
-            st.session_state.action = 'view'
-            st.experimental_rerun()
+    if dataset is not None:
+        st.session_state.dataset_id = dataset.id
+        st.session_state.action = 'view'
+        st.experimental_rerun()
 
 
 def main_app(api: API):
@@ -408,11 +450,15 @@ def main_app(api: API):
         delete_view(api=api)
     elif action == 'sample':
         sample_view(api=api)
+    elif action == 'auxiliary':
+        auxiliary_upload_view(api=api)
 
 
 if __name__=='__main__':
-    def run(db_name='data.db'):
-        api = API(db_name=db_name)
+    st.set_page_config(page_title='Data Manager', layout='wide')
+    
+    def run(data_path=None, db_name='data.db'):
+        api = API(data_path=data_path, db_name=db_name)
         main_app(api=api)
 
     import fire
