@@ -1,4 +1,5 @@
 from typing import Tuple
+from io import StringIO
 
 import streamlit as st
 import numpy as np
@@ -61,6 +62,27 @@ to the empirical variogram. This will also let you use the model parameterizatio
 Using `skgstat` you can choose between different methods for fitting.
 """
 
+
+def _arr_to_dat(arr: np.array) -> bytes:
+    """Converts an array to a string representation in .DAT format"""
+    # create the buffer
+    buf = StringIO()
+
+    # determine format
+    fmt = '%.1f'
+    if abs(np.mean(arr)) < 1:
+        fmt = '%.3f'
+    if abs(np.mean(arr)) < 1e-3:
+        fmt = '%.5f'
+
+    # write
+    np.savetxt(buf, arr, fmt=fmt)
+
+    # encode and return
+    buf.seek(0)
+    return buf.read().encode('utf-8')
+
+
 def _code_sample(**kwargs):
     code = """import skgstat as skg\nfrom skgstat_uncertainty.api import API\n\napi = API()\n"""
     
@@ -120,13 +142,13 @@ def check_story_mode(api: API):
         st.stop()
 
 
-def binning(api: API, dataset: DataUpload, expander = st.sidebar) -> None:
+def binning(api: API, dataset: DataUpload, expander = st.sidebar, no_story: bool = False) -> None:
     """Learn about binning methods and set the parameters to session state"""
     # base variogram is always needed
     v = dataset.base_variogram()
 
     # break out
-    if not st.session_state.get('story_mode', True) or hasattr(st.session_state, 'bin_method'):
+    if not st.session_state.get('story_mode', True) or hasattr(st.session_state, 'bin_method') or no_story:
         expander.selectbox('Binning Method', options=list(BIN_FUNC.keys()), format_func=lambda k: BIN_FUNC.get(k), key='bin_method')
         opt = expander.selectbox('Maxlag Method', options=['none', 'median', 'mean', 'absolute'], format_func=lambda k: k.capitalize())
         if opt == 'none':
@@ -181,10 +203,10 @@ def binning(api: API, dataset: DataUpload, expander = st.sidebar) -> None:
         st.stop()
 
 
-def estimator(api: API, dataset: DataUpload, expander = st.sidebar) -> None:
+def estimator(api: API, dataset: DataUpload, expander = st.sidebar, no_story: bool = False) -> None:
     """Learn about SciKit-GStat's implemented estimators and set the parameters to session state"""
     # break out
-    if not st.session_state.get('story_mode', True) or hasattr(st.session_state, 'estimator'):
+    if not st.session_state.get('story_mode', True) or hasattr(st.session_state, 'estimator') or no_story:
         expander.selectbox('Semivariance estimator', options=list(ESTIMATORS.keys()), format_func=lambda k: ESTIMATORS.get(k), key='estimator')
         return
     
@@ -219,10 +241,10 @@ def estimator(api: API, dataset: DataUpload, expander = st.sidebar) -> None:
         st.stop()
 
 
-def fit_method(api: API, dataset: DataUpload, expander = st.sidebar) -> None:
+def fit_method(api: API, dataset: DataUpload, expander = st.sidebar, no_story: bool = False) -> None:
     """Learn about SciKit-GStat's implemented fit methods and set the parameters to session state"""
     # break out
-    if not st.session_state.get('story_mode', True) or hasattr(st.session_state, 'fit_method'):
+    if not st.session_state.get('story_mode', True) or hasattr(st.session_state, 'fit_method') or no_story:
         expander.selectbox('Fit Method', options=list(FIT_METHODS.keys()), format_func=lambda k: FIT_METHODS.get(k), key='fit_method')
         expander.checkbox('Use nugget', key='use_nugget')
         return
@@ -286,14 +308,14 @@ def fitting(api: API, dataset: DataUpload, expander = st.sidebar) -> None:
         vario = base_variogram(dataset, fit_method='trf')
 
         if vario.use_nugget:
-            nug, sill = expander.slider('Nugget & Sill', min_value=0.0, max_value=float(np.max(vario.experimental)), value=(0.0, float(np.mean(vario.experimental))))
+            nug, sill = expander.slider('Nugget & Sill', min_value=0.0, max_value=float(np.nanmax(vario.experimental)), value=(0.0, float(np.nanmean(vario.experimental))))
             st.session_state.nugget = nug
             st.session_state.sill = sill
         else:
-            sill = expander.slider('Sill', min_value=0.0, max_value=float(np.max(vario.experimental)), value=float(np.mean(vario.experimental)), key='sill')
+            sill = expander.slider('Sill', min_value=0.0, max_value=float(np.nanmax(vario.experimental)), value=float(np.nanmean(vario.experimental)), key='sill')
         
         # add the range
-        expander.slider('Effective range', min_value=float(vario.bins[0]), max_value=float(vario.bins[-1]), value=float(np.mean(vario.bins)), key='range')
+        expander.slider('Effective range', min_value=float(vario.bins[0]), max_value=float(vario.bins[-1]), value=float(np.nanmean(vario.bins)), key='range')
     else:
         # handle fit sigma
         SIG = dict(none='Equal Weights', linear='Linear decreasing', exp='Logarithmic decreasing', sq='Exponential decreasing')
@@ -359,7 +381,7 @@ def variogram(api: API, dataset: DataUpload, always_plot: bool = True) -> None:
             st.stop()
 
 
-@st.experimental_memo
+#@st.experimental_memo
 def _apply_kriging(_dataset: DataUpload, vario_params: dict, grid_resolution: int = 100, **kwargs) -> Tuple[np.ndarray, np.ndarray]:
     # get the variogram
     vario = base_variogram(_dataset, **vario_params)
@@ -378,21 +400,7 @@ def _apply_kriging(_dataset: DataUpload, vario_params: dict, grid_resolution: in
 
 def kriging(api: API, dataset: DataUpload, expander = st.sidebar) -> None:
     """Enable a kriging interface"""
-    # check if kriging is enabled
-    if st.session_state.get('kriging_enabled', False):
-        ok = expander.button('Disable kriging')
-        if ok:
-            st.session_state.kriging_enabled = False
-            st.experimental_rerun()
-    else:
-        st.sidebar.write('You can run a basic kriging interpolation')
-        run = expander.button('Enable')
-        if run:
-            st.session_state.kriging_enabled = True
-            st.experimental_rerun()
-        else:
-            return
-    
+
     # get kriging parameters
     krig_method = expander.selectbox('Kriging method', options=[m for m in KRIGING_METHODS.keys() if m != 'external'], format_func=lambda k: KRIGING_METHODS.get(k))
     krig_kw = {}
@@ -414,11 +422,13 @@ def kriging(api: API, dataset: DataUpload, expander = st.sidebar) -> None:
     l.markdown('### Kriging interpolation')
     fig = go.Figure(go.Heatmap(z=field, colorscale='Earth_r'))
     l.plotly_chart(fig, use_container_width=True)
+    l.download_button('Download kriging data', _arr_to_dat(field), file_name='kriging.dat', mime='text/plain')
 
     # center
     c.markdown('### Kriging error variance')
     fig = go.Figure(go.Heatmap(z=sigma))
     c.plotly_chart(fig, use_container_width=True)
+    c.download_button('Download kriging error variance', _arr_to_dat(sigma), file_name='kriging_error_variance.dat', mime='text/plain')
 
     # right
     r.markdown('### Python sample code')
@@ -428,12 +438,13 @@ def kriging(api: API, dataset: DataUpload, expander = st.sidebar) -> None:
     c += "y = np.linspace(vario.coordinates[:,1].min(), vario.coordinates[:,1].max(), 100)\n"
     c += "\n#Apply\nfield, sigma = krige.structured((x, y))\n"
     r.code(c, language='python')
+    r.download_button('Download sample code', c.encode('utf-8'), file_name='kriging.py', mime='text/plain')
     
 
 def base_variogram(dataset: DataUpload, **kwargs) -> skg.Variogram:
     # get the needed parameters
     bin_func = kwargs.get('bin_method', st.session_state.get('bin_method'))
-    n_lags = kwargs.get('n_lags', st.session_state.get('n_lags', 10))
+    n_lags = int(kwargs.get('n_lags', st.session_state.get('n_lags', 10)))
     maxlag = kwargs.get('maxlag', st.session_state.get('maxlag'))
     estimator = kwargs.get('estimator', st.session_state.get('estimator'))
     model = kwargs.get('model', st.session_state.get('model'))
